#################################################################
#
#    R-script used for performing simulations 
#    
#    Analysis of Molecular Variance (AMOVA) for autopolyploids
#
#    Patrick G. Meirmans & Shenglin Liu 
#    Frontiers in Ecology & Evolution
#    comments & questions:p.g.meirmans@uva.nl
#
#################################################################




library(vegan)

#function to calculate Hs, Ht and Gst from a matrix of allele frequencies
sum.stats = function(freqs){
	num=dim(freqs)[1]
	
	squaresS = apply(freqs, c(1,2), function(x) sum(x^2))
	meanT = apply(freqs,c(2,3), mean)
	squaresT = apply(meanT, 1, function(x) sum(x^2))

	#average per locus
	Hs = 1 - apply(squaresS,2,mean)
	Ht = 1 - squaresT
	
	#take averages, only for loci with variation
	Hs = mean(Hs[Ht>0])
	Ht = mean(Ht[Ht>0])
	
	#Dst and Gst, do use correction for number of populations
	Dpst = (num / (num-1)) * (Ht - Hs)
	Hpt = Hs + Dpst
	Gst = Dpst / Hpt
	
	return(c(Hs, Ht, Gst))
}


#set the parameters, 
numPops = 20         #number of populations
numArch = 2          #number of archipelagoes, these are equally sized, as far as possible
numLoci = 100        #number of independent loci (serve as replicates)
popSize = 100        #number of individuals per population
ploidy = 6           #ploidy level of the populations, maximum possible is 8
mig = 0.1            #m1: migration among populations within archipelagoes
archMig = 0.01       #m2: migration among archipelagoes
numAll = 100         #maximum number of alleles per locus under a k-alleles mutation model
mut = 0.0001         #mutation rate under a k-alleles model
sampleSize = 100     #the number of individuals to sample at the end of the simulation
generations = 20000  #the number of generations to run the model
which = c("full", "partial") #whether to save full genotypes or phenotypes without dosage information (can also be "none")


#determine which populations belong to which archipelago
archipelagoes = sort(rep(1:numArch, length.out=numPops))
archSizes = rep(0, numArch)
for(a in 1:numArch){
	archSizes[a] = sum(archipelagoes == a)
}
#make a pairwise migration matrix
migmatrix = array(0, c(numPops, numPops))
for(p in 1:numPops){
	for(q in 1:numPops){
		if(archipelagoes[p] == archipelagoes[q]){
			migmatrix[p,q] = migmatrix[q,p] = mig / (archSizes[archipelagoes[p]]-1)
		}else{
			migmatrix[p,q] = migmatrix[q,p] = archMig / (numPops - archSizes[archipelagoes[p]]) #when archSizes is unbalanced, the matrix is nonsymmetrical
		}
	}
}

#fill in the diagonals correctly
for(p in 1:numPops){
	migmatrix[p,p] = 0
	migmatrix[p,p] = 1-sum(migmatrix[p,])
}

#scale mutation rate by numAll to prevent mutating to same allelic state
mut = mut * numAll / (numAll-1)

#the populations are represented by a simple array with integers, initialised with maximum diversity
pops = rmultinom(numPops * numLoci, ploidy*popSize, rep(1/numAll, numAll))
dim(pops) = c(numAll, numPops, numLoci)
pops=aperm(pops, c(2,3,1))

pops = array(0,c(numPops, numLoci, numAll))
for(p in 1:numPops){
	pops[p,1,p]=ploidy*popSize	
}

#initialise variable to save all statistics
allstats = NULL
			
#do generations, this is where the actual simulation takes place
for(g in 1:generations){	
	
	#get expectation of allele frequency after migration
	exp = array(NA,dim(pops)) 
	for(p in 1:numPops){
		exp[p,,] = colSums(pops*migmatrix[,p])
	}
	exp = exp / (ploidy*popSize)
	
	#now some mutation
	exp2 = exp-exp*mut + mut/numAll
	
	#use the expected frequencies for random sampling from a binomial distribution				
	for(p in 1:numPops){
		for(l in 1:numLoci){
			pops[p,l,] = rmultinom(1, ploidy*popSize, exp2[p,l,])
		}
	}
	
	#every 100 generations, calculate some statistics to be able to plot Hs, Ht, and Gst over time to check for convergence
	if(g %% 100 == 0){
		print(g) #print how far we are
		allstats = rbind(allstats, c(g,sum.stats(pops / (ploidy*popSize))))	
	}	
}


#start graphics device, and define some nice-looking colors with funky names
pdf(file = sprintf("Plots_convergence.pdf", numPops), onefile=TRUE, paper="A4r", width=15, height=7) 
cols= c("olivedrab3", "firebrick3", "dodgerblue3", "sienna1", "darkorchid3", "gold1")

#plot to check for convergence
plot(allstats[,1], allstats[,2], ylim=c(0,1), xlab="Generation", ylab="Statistic", pch=NA, font.lab=2, main=sprintf("%dx u%s m%s am%s.gdv", ploidy, mut, mig, archMig))
lines(allstats[,1], allstats[,2], col = cols[1], lwd=2)
lines(allstats[,1], allstats[,3], col = cols[2], lwd=2)
lines(allstats[,1], allstats[,4], col = cols[3], lwd=2)
legend("topright", c("Hs", "Ht", "Fst"), col=cols[1:3], lwd=2)

#switch off graphics device
dev.off()

#write all output to a file
colnames(allstats) = c("gen", "Hs", "Ht", "Fst")
filename = sprintf("allstats_%dx_u%.6f_m%.4f_am%.4f_n%d.txt", ploidy, mut, mig, archMig, sampleSize)
write.table(allstats,filename, quote=FALSE, sep="\t", col.names=TRUE)

#Now create individual genotypes from the populations and write these to a file

#first fill huge array with all genotypes
#no random drawing based on allel frequencies, but random order of known allele counts in pops
genotypes = array(0,dim=c(numPops, numLoci, popSize, ploidy))
for(p in 1:numPops){
	for(l in 1:numLoci){
		loc = rep.int(1:numAll, pops[p,l,])
		loc = sample(loc, size=ploidy*popSize)
		dim(loc) = c(popSize,ploidy)		
		genotypes[p,l,,] = loc
	}
}

#remove dosage information
if("partial" %in% which){
	undosaged = array(0,dim=c(numPops, numLoci, popSize, ploidy))
	for(p in 1:numPops){
		for(l in 1:numLoci){
			for(i in 1:popSize){
				type = genotypes[p,l,i,]
				alleles = as.numeric(names(table(type)))
				len=length(alleles)
				if(len > 1){
					undosaged[p,l,i,1:len] = alleles	
				}else{
					undosaged[p,l,i,] = type	
				}
			}
		}
	}
}	



#now write out to GenoDive file
#optionally sample only first few individuals: sampleSize
#can save full or undosaged (partial) data or both
for(w in which){
	if(w=="full"){
		filename = sprintf("output_%dx_u%.6f_m%.4f_am%.4f_n%d_full.gdv", ploidy, mut, mig, archMig, sampleSize)
		dats = genotypes
	}else if(w=="partial"){
		filename = sprintf("output_%dx_u%.6f_m%.4f_am%.4f_n%d_partial.gdv", ploidy, mut, mig, archMig, sampleSize)
		dats = undosaged
	}else{
		break	
	}
	
	firstlines = sprintf("Data simulated under hierarchical island model, mig=%f, archMig=%f, mut=%f.\n%d\t%d\t%d\t%d\t%d\tgroup1\tgroup2\n", mig, archMig, mut, numPops*sampleSize, numPops, numLoci, ploidy,3)
	cat(firstlines, file= filename)
	cat(sprintf("pop%03d\tgroup%d",1:numPops, archipelagoes), file= filename, sep="\n", append=TRUE)
	for(p in 1:numPops){
		for(i in 1:sampleSize){
			cat(sprintf("%d\tInd%03d_%04d\t",p, p, i), file= filename, sep="\t", append=TRUE)
			#doing this separately for every ploidy level is quicker than looping over loci
			if(ploidy==2){
				cat(sprintf("%03d%03d", dats[p,,i,1], dats[p,,i,2]), file= filename, sep="\t", append=TRUE)
			}else if(ploidy==3){
				cat(sprintf("%03d%03d%03d", dats[p,,i,1], dats[p,,i,2], dats[p,,i,3]), file= filename, sep="\t", append=TRUE)
			}else if(ploidy==4){
				cat(sprintf("%03d%03d%03d%03d", dats[p,,i,1], dats[p,,i,2], dats[p,,i,3], dats[p,,i,4]), file= filename, sep="\t", append=TRUE)
			}else if(ploidy==5){
				cat(sprintf("%03d%03d%03d%03d%03d", dats[p,,i,1], dats[p,,i,2], dats[p,,i,3], dats[p,,i,4], dats[p,,i,5]), file= filename, sep="\t", append=TRUE)
			}else if(ploidy==6){
				cat(sprintf("%03d%03d%03d%03d%03d%03d", dats[p,,i,1], dats[p,,i,2], dats[p,,i,3], dats[p,,i,4], dats[p,,i,5], dats[p,,i,6]), file= filename, sep="\t", append=TRUE)
			}else if(ploidy==7){
				cat(sprintf("%03d%03d%03d%03d%03d%03d%03d", dats[p,,i,1], dats[p,,i,2], dats[p,,i,3], dats[p,,i,4], dats[p,,i,5], dats[p,,i,6], dats[p,,i,7]), file= filename, sep="\t", append=TRUE)
			}else if(ploidy==8){
				cat(sprintf("%03d%03d%03d%03d%03d%03d%03d%03d", dats[p,,i,1], dats[p,,i,2], dats[p,,i,3], dats[p,,i,4], dats[p,,i,5], dats[p,,i,6], dats[p,,i,7], dats[p,,i,8]), file= filename, sep="\t", append=TRUE)
			}else{
				warning("ploidy level too high, no data generated");	
			} 
			cat("\n", file= filename, sep="\t", append=TRUE)
		}
	}
}	
